/* ==========================================================================*/
/* - - - - - - - - - - -  K e y p a r s e r . c  - - - - - - - - - - - - - - */
/* ==========================================================================*/
/*	Created   2010-05-29   M. Wheatley                                       */
/*	Modified  2010-05-29   MW                                                */
/*.........................................................................  */
/* WSPRbeacon   AT91SAM7256 terminal Key parser                              */
///////////////////////////////////////////////////////////////////////////////
//            2011 Copyright (C) Moe Wheatley, AE4JY
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//////////////////////////////////////////////////////////////////////////////

/*---------------------------------------------------------------------------*/
/*------------------------> I N C L U D E S <--------------------------------*/
/*---------------------------------------------------------------------------*/
#include "board.h"
#include "common.h"
#include "iicctrl.h"
#include "Si5338.h"
#include "protocoldefs.h"
#include "keyparser.h"
#include <stdlib.h>
#include "string.h"

/*---------------------------------------------------------------------------*/
/*--------------------> L O C A L   D E F I N E S <--------------------------*/
/*---------------------------------------------------------------------------*/
#define KS_NORM 0	//usart esc decode states
#define KS_ESC 1
#define KS_5B 2		//'['
#define KS_TILDA 3	//'~'

//map some multicode keys
#define KEY_DEL 0x100	//esc[3~
#define KEY_END 0x101	//esc[4~
#define KEY_HOME 0x102	//esc[1~
#define KEY_UP 0x103	//esc[A
#define KEY_DN 0x104	//esc[B
#define KEY_LEFT 0x105	//esc[D
#define KEY_RIGHT 0x106	//esc[C
#define KEY_RET '\r'
#define KEY_BKSP 0x08
#define KEY_BKSPCODE 0x7F
#define KEY_ESC 0x1B


#define MENU_TOP 0
#define MENU_CALLSIGN 1
#define MENU_FREQUENCY 2
#define MENU_POWER 3
#define MENU_INTERVAL 4

#define BUF_SIZE 16



/*---------------------------------------------------------------------------*/
/*-------------------> M E M O R Y  A L L O C A T I O N <--------------------*/
/*---------------------------------------------------------------------------*/
static uint32_t EscState;
static uint32_t MenuState;
static char EditBuf[BUF_SIZE];
static char FreqStrBuf[BUF_SIZE];
static uint8_t EditPos;
static uint8_t EditMin;
static uint8_t EditMax;


/*---------------------------------------------------------------------------*/
/*--------------------------> P R O T O T Y P E S <--------------------------*/
/*---------------------------------------------------------------------------*/
static void ParseKey(uint32_t key);
static int EditCallsign(int ch);
static int EditFrequency(int ch);
static int EditInterval(int ch);
static int EditPower(int ch);
static void PrintStatus();
static int GetFrequencyFromStr( char* str);

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                    ---------------------------                            */
/*                   | K e y p a r s e r I n i t |                           */
/*                    ---------------------------                            */
/*    Called to initialize the Keyparser.                                    */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void KeyParserInit( void )
{
	EscState = KS_NORM;
	if(	CI_OPMODE_SETUP == Settings.OpMode)
		PrintStatus();
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                    -----------------------------                          */
/*                   | P r o c e s s N e w B y t e |                         */
/*                    -----------------------------                          */
/*    Called with new usart data byte to parse from terminal.                */
/*Manages ESC sequencesinto key code.                                        */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void ProcessNewByte (char ch )
{
	if(KEY_BKSPCODE == ch)	//map keyboard bkspace to ascii bkspace code
		ch = KEY_BKSP;
	switch(EscState)
	{
		case KS_NORM:
			if(KEY_ESC==ch)
				EscState = KS_ESC;
			else
				ParseKey(ch);
			break;
		case KS_ESC:
			if('['==ch)
				EscState = KS_5B;
			else
				EscState = KS_NORM;
			if(KEY_ESC==ch)
				ParseKey(ch);
			break;
		case KS_5B:
			EscState = KS_NORM;
			if('A'==ch)
				ParseKey(KEY_UP);
			else if('B'==ch)
				ParseKey(KEY_DN);
			else if('C'==ch)
				ParseKey(KEY_RIGHT);
			else if('D'==ch)
				ParseKey(KEY_LEFT);
			else if('1'==ch)
			{
				ParseKey(KEY_HOME);
				EscState = KS_TILDA;
			}
			else if('3'==ch)
			{
				ParseKey(KEY_DEL);
				EscState = KS_TILDA;
			}
			else if('4'==ch)
			{
				ParseKey(KEY_END);
				EscState = KS_TILDA;
			}
			else
				EscState = KS_NORM;
			break;
		case KS_TILDA:
			EscState = KS_NORM;
			break;
	}
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                     -----------------                                     */
/*                    | P a r s e K e y |                                    */
/*                     -----------------                                     */
/*    Called with new key code to parse.                                     */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
static void ParseKey(uint32_t key)
{
//	printf("0x%X ", key);
	switch(MenuState)
	{
		case MENU_TOP:
			key = toupper(key);
			if('\r' == key)
				PrintStatus();
			else if('C'==key)
				EditCallsign(-1);
			else if('F'==key)
				EditFrequency(-1);
			else if('P'==key)
				EditPower(-1);
			else if('I'==key)
				EditInterval(-1);
			break;
		case MENU_CALLSIGN:
			if( EditCallsign(key) )
			{
				MenuState = MENU_TOP;
				PrintStatus();
			}
			break;
		case MENU_FREQUENCY:
			if( EditFrequency(key) )
			{
				MenuState = MENU_TOP;
				PrintStatus();
			}
			break;
		case MENU_POWER:
			if( EditPower(key) )
			{
				MenuState = MENU_TOP;
				PrintStatus();
			}
			break;
		case MENU_INTERVAL:
			if( EditInterval(key) )
			{
				MenuState = MENU_TOP;
				PrintStatus();
			}
			break;
		default:
			break;

	}
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                     -------------------------                             */
/*                    | E d i t C a l l s i g n |                            */
/*                     -------------------------                             */
/*    Process key while in callsign edit mode.  Init when ch==-1             */
/* Returns 1 if finished edit, 0 otherwise.                                  */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
static int EditCallsign(int ch)
{
int i;
	if(-1 == ch)
	{
		MenuState = MENU_CALLSIGN;
		EditPos = 0;
		for(i=0; i<BUF_SIZE; i++)
			EditBuf[i] = 0;
		while(Settings.CallSign[EditPos])
		{
			EditBuf[EditPos] = Settings.CallSign[EditPos];
			EditPos++;
		}
		printf("\r\n%s",EditBuf);
		return 0;
	}
	else
	{
		switch(ch)
		{
			case KEY_DEL:
			case KEY_BKSP:
			case KEY_LEFT:
				printf("\r          ");
				if(EditPos>0)
					EditBuf[--EditPos] = 0;
				break;
			case KEY_END:
				break;
			case KEY_HOME:
				printf("\r          ");
				EditPos = 0;
				EditBuf[EditPos] = 0;
				break;
			case KEY_UP:
				break;
			case KEY_DN:
				break;
			case KEY_RIGHT:
				break;
			case KEY_ESC:
				strcpy(EditBuf, Settings.CallSign );
				PrintStatus();
				return 1;
				break;
			case KEY_RET:
				strcpy(Settings.CallSign, EditBuf);
				printf("\r\n%s",EditBuf);
				for(i=0; i<=strlen(Settings.CallSign); i++)
					WriteFRAM(EE_CALLSIGN+i, Settings.CallSign[i]);
				return 1;
				break;
			default:
				if( (EditPos < 10) && ( (isalnum(ch)) || (ch == '/')) )
				{
					ch = toupper(ch);
					EditBuf[EditPos++] = ch;
					EditBuf[EditPos] = 0;
				}
				break;
		}
		printf("\r%s",EditBuf);
	}
	return 0;
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                     ---------------------------                           */
/*                    | E d i t F r e q u e n c y |                          */
/*                     ---------------------------                           */
/*    Process key while in frequncy edit mode.  Init when ch==-1             */
/* Returns 1 if finished edit, 0 otherwise.                                  */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
static int EditFrequency(int ch)
{
int f;
	if(-1 == ch)
	{
		MenuState = MENU_FREQUENCY;
		for(f=0; f<BUF_SIZE; f++)
			EditBuf[f] = 0;
		GetFrequencyStr(FREQBASE_TBL[Settings.BandIndex] + Settings.FreqOffset[Settings.BandIndex], EditBuf);
		EditPos = strlen(EditBuf);
		EditMax = EditPos;
		EditMin = EditMax - 3;	//only edit last three digits
		printf("\r\n%s",EditBuf);
		return 0;
	}
	else
	{
		switch(ch)
		{
			case KEY_DEL:
			case KEY_BKSP:
			case KEY_LEFT:
				if( (EditPos <= EditMax)&& (EditPos > EditMin) )
				{
					printf("\r          ");
					EditBuf[--EditPos] = 0;
				}
				break;
			case KEY_END:
				break;
			case KEY_HOME:
				break;
			case KEY_UP:
				break;
			case KEY_DN:
				break;
			case KEY_RIGHT:
				break;
			case KEY_ESC:
				PrintStatus();
				return 1;
				break;
			case KEY_RET:
				f = GetFrequencyFromStr(EditBuf);
				f = f - FREQBASE_TBL[Settings.BandIndex];
				if(f < FREQ_MIN)
					f = FREQ_MIN;
				if(f > FREQ_MAX)
					f = FREQ_MAX;
				Settings.FreqOffset[Settings.BandIndex] = f;
				Write32FRAM(EE_FREQOFFSET + (4*Settings.BandIndex), Settings.FreqOffset[Settings.BandIndex]);
				SetFreq( FREQBASE_TBL[Settings.BandIndex] + Settings.FreqOffset[Settings.BandIndex]);
				return 1;
				break;
			default:
				if( (EditPos < EditMax) && isdigit(ch) && (EditPos >= EditMin) )
				{
					EditBuf[EditPos++] = ch;
					EditBuf[EditPos] = 0;
				}
				break;
		}
		printf("\r%s",EditBuf);
	}
	return 0;
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                     -------------------------                             */
/*                    | E d i t I n t e r v a l |                            */
/*                     -------------------------                             */
/*    Process key while in Interval edit mode.  Init when ch==-1             */
/* Returns 1 if finished edit, 0 otherwise.                                  */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
static int EditInterval(int ch)
{
int i;
	if(-1 == ch)
	{
		MenuState = MENU_INTERVAL;
		for(i=0; i<BUF_SIZE; i++)
			EditBuf[i] = 0;
		i = Settings.WsprInterval;
		sprintf(EditBuf, "%d", i);
		EditPos = strlen(EditBuf);
		EditMax = 3;
		EditMin = 0;
		printf("\r\n%s",EditBuf);
		return 0;
	}
	else
	{
		switch(ch)
		{
			case KEY_DEL:
			case KEY_BKSP:
			case KEY_LEFT:
				if( (EditPos <= EditMax)&& (EditPos > EditMin) )
				{
					printf("\r          ");
					EditBuf[--EditPos] = 0;
				}
				break;
			case KEY_END:
				break;
			case KEY_HOME:
				break;
			case KEY_UP:
				break;
			case KEY_DN:
				break;
			case KEY_RIGHT:
				break;
			case KEY_ESC:
				PrintStatus();
				return 1;
				break;
			case KEY_RET:
				i = atoi(EditBuf);
				 i &= 0xFFFFFFFE;	//force to be even
				if(i < INTERVAL_MIN)
					i = INTERVAL_MIN;
				if(i > INTERVAL_MAX)
					i = INTERVAL_MAX;
				Settings.WsprInterval = i;
				WriteFRAM(EE_WSPRINTERVAL, Settings.WsprInterval);
				return 1;
				break;
			default:
				if( (EditPos < EditMax) && isdigit(ch) && (EditPos >= EditMin) )
				{
					EditBuf[EditPos++] = ch;
					EditBuf[EditPos] = 0;
				}
				break;
		}
		printf("\r%s",EditBuf);
	}
	return 0;
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                     -------------------                                   */
/*                    | E d i t P o w e r |                                  */
/*                     -------------------                                   */
/*    Process key while in Power edit mode.  Init when ch==-1                */
/* Returns 1 if finished edit, 0 otherwise.                                  */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
static int EditPower(int ch)
{
int i;
	if(-1 == ch)
	{
		MenuState = MENU_POWER;
		for(i=0; i<BUF_SIZE; i++)
			EditBuf[i] = 0;
		i = Settings.WsprPower;
		sprintf(EditBuf, "%d", i);
		EditPos = strlen(EditBuf);
		EditMax = 2;
		EditMin = 0;
		printf("\r\n%s",EditBuf);
		return 0;
	}
	else
	{
		switch(ch)
		{
			case KEY_DEL:
			case KEY_BKSP:
			case KEY_LEFT:
				if( (EditPos <= EditMax)&& (EditPos > EditMin) )
				{
					printf("\r          ");
					EditBuf[--EditPos] = 0;
				}
				break;
			case KEY_END:
				break;
			case KEY_HOME:
				break;
			case KEY_UP:
				break;
			case KEY_DN:
				break;
			case KEY_RIGHT:
				break;
			case KEY_ESC:
				PrintStatus();
				return 1;
				break;
			case KEY_RET:
				i = atoi(EditBuf);
				if(i < POWER_MIN)
					i = POWER_MIN;
				if(i > POWER_MAX)
					i = POWER_MAX;
				Settings.WsprPower = i;
				WriteFRAM(EE_WSPRPOWER, Settings.WsprPower);
				return 1;
				break;
			default:
				if( (EditPos < EditMax) && isdigit(ch) && (EditPos >= EditMin) )
				{
					EditBuf[EditPos++] = ch;
					EditBuf[EditPos] = 0;
				}
				break;
		}
		printf("\r%s",EditBuf);
	}
	return 0;
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                     -----------------------                               */
/*                    | P r i n t S t a t u s |                              */
/*                     -----------------------                               */
/*    Print Status when return key is pressed.                               */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
static void PrintStatus()
{
	printf("\r\nCall = %s",Settings.CallSign);
	GetFrequencyStr(FREQBASE_TBL[Settings.BandIndex] + Settings.FreqOffset[Settings.BandIndex], FreqStrBuf);
	printf("\r\nFreq = %s MHz", FreqStrBuf);
	printf("\r\nPower = %d dBm", (int)Settings.WsprPower);
	printf("\r\nInterval = %d Min", (int)Settings.WsprInterval);
	printf("\r\nPress 'C', 'F', 'P', or 'I' to Edit");
	printf("\r\nPress ESC twice to abort edit mode");
	printf("\r\nPress Enter to accept edit\r\n>");
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                  -------------------------------                          */
/*                 | G e t F r e q u e n c y S t r |                         */
/*                  -------------------------------                          */
/*    Create Frequency in MHz format and put in users array.                 */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void GetFrequencyStr(int Freq, char* str)
{
int Mhz;
int Hz;
char s[8];
	Mhz = Freq/1000000;
	Hz = Freq%1000000;
	s[0] = Hz/100000 + '0';
	Hz = Hz%100000;
	s[1] = Hz/10000 + '0';
	Hz = Hz%10000;
	s[2] = Hz/1000 + '0';
	Hz = Hz%1000;
	s[3] = Hz/100 + '0';
	Hz = Hz%100;
	s[4] = Hz/10 + '0';
	Hz = Hz%10;
	s[5] = Hz + '0';
	s[6] = 0;
	sprintf(str, "%d.%s", Mhz, s);
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                ---------------------------------------                    */
/*               | G e t F r e q u e n c y F r o m S t r |                   */
/*                ---------------------------------------                    */
/*    Get Frequency from string in MHz format from callers array.            */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
static int GetFrequencyFromStr( char* str)
{
int Freq = atoi(str)*1000000;
int mul = 100000;
char* s = strchr(str, '.') + 1;
	if(s)
	{
		while(*s && mul)
		{
			Freq = Freq + mul *(*s++ - '0');
			mul/=10;
		}
	}
	return Freq;
}
